//
//  FILE NAME:
//      GameSkeletonContainer.cpp
//
//  AUTHOR(S):
//      Rafal Kocisz <rafal.kocisz@gmail.com>
//

#include "FpsCounter.h"
#include "GameSkeletonContainer.h"

#include <coemain.h>
#include <eikenv.h>
#include <gameskeleton.rsg>

const TInt KFpsCounterIntervalInSeconds = 1;
const TInt KOneSecondInMicroSeconds = 1000000;
const TInt KDesiredFramesPerSecond
	= KOneSecondInMicroSeconds / 35;
const TInt KFpsMessageMaxLength = 20;
const TInt KFpsMessagePosX = 0;
const TInt KFpsMessagePosY = 20;

_LIT( KFpsMessageFormatString, "%d FPS" );

void CGameSkeletonContainer::FocusChanged(
    TDrawNow aDrawNow )
    {
    if ( IsFocused() )
        {
        StartTickGenerator();
        }
    else
        {
        if ( iTickGenerator->IsActive() )
            {
            StopTickGenerator();
            }
        }

    if ( aDrawNow )
        {
        DrawNow();
        }
    }

void CGameSkeletonContainer::ConstructL( const TRect& aRect )
    {
    CreateWindowL();
    SetBlank();
    SetRect( aRect );

    iBackupBitmap
        = new ( ELeave ) CWsBitmap( iCoeEnv->WsSession() );
    iBackupBitmap->Create(
        Rect().Size(),
        iEikonEnv->DefaultDisplayMode() );

    iBitmapDevice
        = CFbsBitmapDevice::NewL( iBackupBitmap );
    User::LeaveIfError(
        iBitmapDevice->CreateContext( iBitmapGc ) );

    iTickGenerator
        = CPeriodic::NewL( CActive::EPriorityStandard );

    iFpsCounter = CFpsCounter::NewL(
        KFpsCounterIntervalInSeconds );

    ActivateL();
    }

CGameSkeletonContainer::~CGameSkeletonContainer()
    {
    delete iFpsCounter;
    StopTickGenerator();
    delete iTickGenerator;
    delete iBitmapGc;
    delete iBitmapDevice;
    delete iBackupBitmap;
    }

void CGameSkeletonContainer::SizeChanged()
    {
    }

TInt CGameSkeletonContainer::CountComponentControls() const
    {
    return 0;
    }

CCoeControl* CGameSkeletonContainer::ComponentControl(
    TInt /*aIndex*/ ) const
    {
    return NULL;
    }

void CGameSkeletonContainer::StartTickGenerator()
    {
    if ( !iTickGenerator->IsActive() )
        {
        iTickGenerator->Start(
            KDesiredFramesPerSecond,
            KDesiredFramesPerSecond,
            TCallBack( CGameSkeletonContainer::Tick,
                       this ) );
        iTickStart.HomeTime();
        }
    }

void CGameSkeletonContainer::StopTickGenerator()
    {
    iTickGenerator->Cancel();
    }

void CGameSkeletonContainer::Draw( const TRect& aRect )
    const
    {
    CWindowGc& gc = SystemGc();
    gc.BitBlt( aRect.iTl, iBackupBitmap, aRect );
    }

TInt CGameSkeletonContainer::Tick( TAny* aCallback )
    {
    ASSERT( aCallback );

    CGameSkeletonContainer* gameSkeletonContainer =
        static_cast< CGameSkeletonContainer* >( aCallback );

    gameSkeletonContainer->OnTick();

    return ETrue;
    }

void CGameSkeletonContainer::OnTick()
    {
    iTickStop.HomeTime();

    TInt64 dt = iTickStart.Int64() - iTickStop.Int64();
    iTickStart = iTickStop;

    Update( dt );
    Draw( *iBitmapGc );

    DrawNow();

    iFpsCounter->Update();
    }

void CGameSkeletonContainer::Draw(
    CGraphicsContext& aGc ) const
    {
    aGc.Reset();
    aGc.SetBrushColor( KRgbBlack );
    aGc.SetBrushStyle( CGraphicsContext::ESolidBrush );
    aGc.DrawRect( Rect() );

    TBuf< KFpsMessageMaxLength > fpsMessage;
    fpsMessage.Format( KFpsMessageFormatString,
                       iFpsCounter->Fps() );

    aGc.UseFont( iEikonEnv->NormalFont() );
    aGc.SetPenColor( KRgbWhite );
    aGc.DrawText( fpsMessage,
                  TPoint( KFpsMessagePosX,
                          KFpsMessagePosY ) );
    aGc.DiscardFont();
    }

void CGameSkeletonContainer::Update( TInt64 /*aDt*/ )
    {
    }

TKeyResponse CGameSkeletonContainer::OfferKeyEventL(
    const TKeyEvent& aKeyEvent,
    TEventCode aType )
    {
    TKeyResponse response = EKeyWasConsumed;
    TKeyState input = KKeyNull;

    switch ( aKeyEvent.iScanCode )
        {
        case EStdKeyDevice0: input = KKeySoftLeft; break;
        case EStdKeyDevice1: input = KKeySoftRight; break;
        case EStdKeyUpArrow: input = KKeyUp; break;
        case EStdKeyDownArrow: input = KKeyDown; break;
        case EStdKeyLeftArrow: input = KKeyLeft; break;
        case EStdKeyRightArrow: input = KKeyRight; break;
        case EStdKeyDevice3: input = KKeyFire; break;
        case '0': input = KKey0; break;
        case '1': input = KKey1; break;
        case '2': input = KKey2; break;
        case '3': input = KKey3; break;
        case '4': input = KKey4; break;
        case '5': input = KKey5; break;
        case '6': input = KKey6; break;
        case '7': input = KKey7; break;
        case '8': input = KKey8; break;
        case '9': input = KKey9; break;
        default: response = EKeyWasNotConsumed; break;
        }

    if ( EEventKeyDown == aType )
        {
        iKeyState |= input;
        }
    else if ( EEventKeyUp == aType )
        {
        iKeyState &= ~input;
        }

    return response;
    }

// Eof
